--[[
	Auc-Searcher-Transmogs
	Version: 1.0
	Date: 2016-11-11
	URL: TBD

	This is a plugin module for the SearchUI that assists in searching for unknown transmogs.

	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit license to use this AddOn with these facilities
		since that is its designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

if not AucSearchUI then return end
local lib, parent, private = AucSearchUI.NewSearcher("Transmogs")
if not lib then return end
--local aucPrint,decode,_,_,replicate,_,_,_,_,debugPrint,fill = AucAdvanced.GetModuleLocals()
local get, set, default, Const, resources = parent.GetSearchLocals()
local GetItemInfo = GetItemInfo
local CIMILoaded = false
lib.tabname = "Transmogs"
-- Set our defaults
default("transmogs.customlevel", false)
default("transmogs.currentonly", false)
default("transmogs.maxlevel", Const.MAXUSERLEVEL)
default("transmogs.maxprice.enable", false)
default("transmogs.maxprice", 10000000)
-- set vendor defaults
default("transmogs.vendor.enabled", true)
default("transmogs.vendor.loss.max", 20000)
default("transmogs.vendor.loss.pct", 20)
default("transmogs.vendor.allow.bid", true)
default("transmogs.vendor.allow.buy", true)
--set market defaults
default("transmogs.market.enabled", false)
default("transmogs.market.maxprice", 400000)
default("transmogs.market.pct", 100)
default("transmogs.market.seen.check", false)
default("transmogs.market.seen.min", 10)
default("transmogs.market.allow.bid", true)
default("transmogs.market.allow.buy", true)
default("transmogs.market.model", "market")
--set disenchant defaults
default("transmogs.disenchant.enabled", false)
default("transmogs.disenchant.loss.max", 20000)
default("transmogs.disenchant.loss.pct", 20)
default("transmogs.disenchant.level.custom", false)
default("transmogs.disenchant.level.min", 0)
default("transmogs.disenchant.level.max", Const.MAXSKILLLEVEL)
default("transmogs.disenchant.allow.bid", true)
default("transmogs.disenchant.allow.buy", true)
default("transmogs.disenchant.model", "Enchantrix")

function private.doValidation()
	if not resources.isValidPriceModel(get("transmogs.disenchant.model")) then
		message("Disenchant Searcher Warning!\nCurrent price model setting ("..get("transmogs.disenchant.model")..") is not valid. Select a new price model")
	elseif not resources.isValidPriceModel(get("transmogs.market.model")) then
		message("Disenchant Searcher Warning!\nCurrent price model setting ("..get("transmogs.market.model")..") is not valid. Select a new price model")
	elseif not IsAddOnLoaded("CanIMogIt") then
		message("Transmogs Searcher Warning!\nCanIMogIt not detected\nThis searcher will not function until CanIMogIt is loaded")
	else
		private.doValidation = nil
	end
end

-- This function is automatically called from AucSearchUI.NotifyCallbacks
function lib.Processor(event, subevent)
	if event == "selecttab" then
		if subevent == lib.tabname and private.doValidation then
			private.doValidation()
		end
	end
end

-- This function is automatically called when we need to create our search parameters
function lib:MakeGuiConfig(gui)
	lib.MakeGuiConfig = nil
	-- Get our tab and populate it with our controls
	local id = gui:AddTab(lib.tabname, "Searchers")

	-- Add the help
	gui:AddSearcher("Transmogs", "Search for items that you have not yet learned the appearance of", 100)
	gui:AddHelp(id, "transmogs searcher",
		"What does this searcher do?",
		"This searcher finds cheap items that you have not yet learned the appearance of. That can be vendored for little loss or disencheanted.")

	gui:AddControl(id, "Header",     0,      "Transmogs search criteria")
-- add general options.
	local last = gui:GetLast(id)
	gui:AddControl(id, "Label",          0, 1, nil, "Up to my level or")
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.15, 1, "transmogs.customlevel", "custom level.")
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.42, 1, "transmogs.currentonly", "Current class only")
	last = gui:GetLast(id)
	gui:AddControl(id, "NumeriSlider",      0, 2, "transmogs.maxlevel", 1, Const.MAXUSERLEVEL, 1, "Max level")
	gui:AddControl(id, "Checkbox",          0, 1, "transmogs.maxprice.enable", "Enable individual maximum price:")
	gui:AddTip(id, "Limit the maximum amount you want to spend with this searcher")
	gui:SetLast(id, last)
	gui:AddControl(id, "MoneyFramePinned",  0.42, 2, "transmogs.maxprice", 1, Const.MAXBIDPRICE, "Maximum Price")
-- add vendor options
	gui:AddControl(id, "Subhead",           0,    "Vendor search options")
	last = gui:GetLast(id)
	gui:AddControl(id, "Checkbox",          0.22, 1, "transmogs.vendor.enabled", "Enabled")
	gui:SetLast(id, last)
	gui:AddControl(id, "MoneyFramePinned",  0, 1, "transmogs.vendor.loss.max", 1, Const.MAXBIDPRICE, "Maximum loss")
	gui:AddControl(id, "NumeriSlider",      0, 1, "transmogs.vendor.loss.pct", 0, 2000, 1, "Max loss %%")
	local templast = gui:GetLast(id)
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.42, 1, "transmogs.vendor.allow.bid", "Allow Bids")
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.56, 1, "transmogs.vendor.allow.buy", "Allow Buyouts")
	
-- add market options
	gui:SetLast(id, templast)
	gui:AddControl(id, "Subhead",           0,    "Market price search options")
	last = gui:GetLast(id)
	gui:AddControl(id, "Checkbox",          0.22, 1, "transmogs.market.enabled", "Enabled")
	gui:SetLast(id, last)
	gui:AddControl(id, "MoneyFramePinned",  0, 1, "transmogs.market.maxprice", 1, Const.MAXBIDPRICE, "Maximum Price")
	gui:AddControl(id, "NumeriSlider",      0, 1, "transmogs.market.pct", 1, 300, 1, "%% of value")
	gui:AddControl(id, "Checkbox",          0, 1, "transmogs.market.seen.check", "Check Seen count")
	gui:AddControl(id, "NumeriSlider",            0, 1, "transmogs.market.seen.min", 1, 100, 1, "Min seen")
	templast = gui:GetLast(id)
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.42, 1, "transmogs.market.allow.bid", "Allow Bids")
	gui:SetLast(id, last)
	gui:AddControl(id, "Checkbox",          0.56, 1, "transmogs.market.allow.buy", "Allow Buyouts")

	gui:AddControl(id, "Subhead",           0.42,    "Price Valuation Method:")
	gui:AddControl(id, "Selectbox",         0.42, 1, resources.selectorPriceModels, "transmogs.market.model")
	gui:AddTip(id, "The pricing model that is used to work out the calculated value of items at the Auction House.")

-- add disenchant options
	if resources.isEnchantrixLoaded then
		gui:SetLast(id, templast)
		gui:AddControl(id, "Subhead",           0,    "Disenchant search options")
		last = gui:GetLast(id)
		gui:AddControl(id, "Checkbox",          0.22, 1, "transmogs.disenchant.enabled", "Enabled")
		gui:SetLast(id, last)
		gui:AddControl(id, "MoneyFramePinned",  0, 1, "transmogs.disenchant.loss.max", 1, Const.MAXBIDPRICE, "Maximum loss")
		gui:AddControl(id, "NumeriSlider",            0, 1, "transmogs.disenchant.loss.pct", 1, 2000, 1, "Max loss %%")
		gui:AddControl(id, "Checkbox",          0, 1, "transmogs.disenchant.level.custom", "Use custom levels")
		gui:AddControl(id, "Slider",            0, 2, "transmogs.disenchant.level.min", 0, Const.MAXSKILLLEVEL, 25, "Minimum skill: %s")
		gui:AddControl(id, "Slider",            0, 2, "transmogs.disenchant.level.max", 25, Const.MAXSKILLLEVEL, 25, "Maximum skill: %s")
		
		gui:SetLast(id, last)
		gui:AddControl(id, "Checkbox",          0.42, 1, "transmogs.disenchant.allow.bid", "Allow Bids")
		gui:SetLast(id, last)
		gui:AddControl(id, "Checkbox",          0.56, 1, "transmogs.disenchant.allow.buy", "Allow Buyouts")
		gui:AddControl(id, "Subhead",           0.42,    "Price Valuation Method:")
		gui:AddControl(id, "Selectbox",         0.42, 1, resources.selectorPriceModelsEnx, "transmogs.disenchant.model")
		gui:AddTip(id, "The pricing model that is used to work out the calculated value of items at the Auction House.")
	else
		gui:AddControl(id, "Label",          0, 1, nil, "Enchantrix not loaded, disenchant searching disabled.")
		set("transmogs.disenchant.enabled", false)
	end
end

local function isWanted(item)
	local itemLink = item[Const.LINK]
	if not CanIMogIt:IsTransmogable(itemLink) then
		return false, "not transmoggable"
	end
	if CanIMogIt:PlayerKnowsTransmog(itemLink) then
		return false, "already known"
	end
	if get("transmogs.currentonly") and not CanIMogIt:CharacterCanLearnTransmog(itemLink) then
		return false, "not for current char"
	end
	local maxlevel = (get("transmogs.customlevel") and get("transmogs.maxlevel")) or UnitLevel("player")
	if CanIMogIt:GetItemMinLevel(itemLink) and (maxlevel < CanIMogIt:GetItemMinLevel(itemLink)) then
		return false, "item min lvl too high"
	end
	return true
end

local function tryVendor(item)
	local bidprice, buyprice = item[Const.PRICE], item[Const.BUYOUT]
	local maxprice = get("transmogs.maxprice.enable") and get("transmogs.maxprice")
	if buyprice <= 0 or not get("transmogs.vendor.allow.buy") or (maxprice and buyprice > maxprice) then
		buyprice = nil
	end
	if not get("transmogs.vendor.allow.bid") or (maxprice and bidprice > maxprice) then
		bidprice = nil
	end
	if not (bidprice or buyprice) then
		return false, "Does not meet transmogs maxprice"
	end

	local _,_,_,_,_,_,_,_,_,_,market = GetItemInfo(item[Const.ITEMID])
	-- If there's no price, then we obviously can't sell it, ignore!
	if not market or market == 0 then
		return false, "No vendor price"
	end
	market = market * item[Const.COUNT]

	local value = market * (100+get("transmogs.vendor.loss.pct")) / 100
	local value2 = market + get("transmogs.vendor.loss.max")
	if value > value2 then
		value = value2
	end
	if buyprice and buyprice <= value then
		return "buy", value
	elseif bidprice and bidprice <= value then
		return "bid", value
	end
	return false, "Too expensive"
end

local function tryDisenchant(item)
	if not resources.isEnchantrixLoaded then
		return false, "Enchantrix not detected"
	end

	local itemQuality = item[Const.QUALITY]
	if itemQuality <= 1 then
		return false, "Item quality too low"
	end
	
	local bidprice, buyprice = item[Const.PRICE], item[Const.BUYOUT]
	local maxprice = get("transmogs.maxprice.enable") and get("transmogs.maxprice")
	if buyprice <= 0 or not get("transmogs.vendor.allow.buy") or (maxprice and buyprice > maxprice) then
		buyprice = nil
	end
	if not get("transmogs.vendor.allow.bid") or (maxprice and bidprice > maxprice) then
		bidprice = nil
	end
	if not (bidprice or buyprice) then
		return false, "Does not meet transmogs maxprice"
	end
	
	local minskill, maxskill
	if get("transmogs.disenchant.level.custom") then
		minskill = get("transmogs.disenchant.level.min")
		maxskill = get("transmogs.disenchant.level.max")
	else
		minskill = 0
		maxskill = Enchantrix.Util.GetUserEnchantingSkill()
	end
	local skillneeded = Enchantrix.Util.DisenchantSkillRequiredForItemLevel(item[Const.ILEVEL], itemQuality)
	if (skillneeded < minskill) or (skillneeded > maxskill) then
		return false, "Skill not high enough to disenchant"
	end
	
	local data = Enchantrix.Storage.GetItemDisenchants(item[Const.LINK])	-- was Const.ITEMID
	if not data then
		return false, "Item not Disenchantable"
	end
	
	local total = data.total
	local market = 0
	if total and total[1] > 0 then
		local totalNumber, totalQuantity = unpack(total)
		local model = get("transmogs.disenchant.model")
		local GetPrice = resources.lookupPriceModel[model]
		for result, resData in pairs(data) do
			if result ~= "total" then
				local resNumber, resYield = unpack(resData)
				local price = GetPrice(model, result)
				if price then
					market = market + price * resYield / totalNumber
				end
			end
		end
	end
	if market <= 0 then
		return false, "No price found"
	end
	
	local value = market * (100+get("transmogs.disenchant.loss.pct")) / 100
	local value2 = market + get("transmogs.disenchant.loss.max")
	if value > value2 then
		value = value2
	end
	if buyprice and buyprice <= value then
		return "buy", value
	elseif bidprice and bidprice <= value then
		return "bid", value
	end
	return false, "Too expensive"
end

local function tryMarket(item)
	local link = item[Const.LINK]
	if not link then
		return false, "No link"
	end
	
	local bidprice, buyprice, link = item[Const.PRICE], item[Const.BUYOUT], item[Const.LINK]
	local maxprice = get("transmogs.market.maxprice")
	if buyprice <= 0 or not get("transmogs.market.allow.buy") or (maxprice and buyprice > maxprice) then
		buyprice = nil
	end
	if not get("transmogs.market.allow.bid") or (maxprice and bidprice > maxprice) then
		bidprice = nil
	end
	if not (bidprice or buyprice) then
		return false, "Higher than max price"
	end
	
	local model = get("transmogs.market.model")
	local value, seen, curModel = resources.lookupPriceModel[model](model, link)
	if not value then
		return false, "No market price"
	end
	local count = item[Const.COUNT]
	value = value * count

	if (get("transmogs.market.seen.check")) and curModel ~= "fixed" then
		if ((not seen) or (seen < get("transmogs.market.seen.min"))) then
			return false, "Seen count too low"
		end
	end

	local value = value*get("transmogs.market.pct")/100
	if value > maxprice then
		value = maxprice
	end
	if buyprice and buyprice <= value then
		return "buy", value
	elseif bidprice and bidprice <= value then
		return "bid", value
	end
	return false, "Higher than percent allows"
end

function lib.Search(item)
	local value
	if not IsEquippableItem(item[Const.ITEMID]) then
		return false, "Not equipment"
	end
	
	if get("transmogs.vendor.enabled") then
		local buyorbid
		buyorbid, value = tryVendor(item)
		if buyorbid then
			local wanted, reason = isWanted(item)
			if not wanted then
				return false, reason
			end
			return buyorbid, value, nil, "Tmog:V"
		end
	end
	
	if get("transmogs.disenchant.enabled") then
		local buyorbid
		buyorbid, value = tryDisenchant(item)
		if buyorbid then
			local wanted, reason = isWanted(item)
			if not wanted then
				return false, reason
			end
			return buyorbid, value, nil, "Tmog:DE"
		end
	end
	
	if get("transmogs.market.enabled") then
		local buyorbid
		buyorbid, value = tryMarket(item)
		if buyorbid then
			local wanted, reason = isWanted(item)
			if not wanted then
				return false, reason
			end
			return buyorbid, value, nil, "Tmog:M"
		end
	end
	
	return false, "None matched"
end

